@@ -38,6 +38,7 @@
 
 #include "proc.h"
 #include "gotwebd.h"
+#include "tmpl.h"
 
 size_t	 fcgi_parse_record(uint8_t *, size_t, struct request *);
 void	 fcgi_parse_begin_request(uint8_t *, uint16_t, struct request *,
@@ -262,12 +263,12 @@ fcgi_parse_params(uint8_t *buf, uint16_t n, struct request *c, uint16_t id)
 				*sd = '\0';
 		}
 
-		if (c->script_name[0] == '\0' &&
-		    val_len < MAX_SCRIPT_NAME &&
-		    name_len == 11 &&
-		    strncmp(buf, "SCRIPT_NAME", 11) == 0) {
-			memcpy(c->script_name, val, val_len);
-			c->script_name[val_len] = '\0';
+		if (c->document_uri[0] == '\0' &&
+		    val_len < MAX_DOCUMENT_URI &&
+		    name_len == 12 &&
+		    strncmp(buf, "DOCUMENT_URI", 12) == 0) {
+			memcpy(c->document_uri, val, val_len);
+			c->document_uri[val_len] = '\0';
 		}
 
 		if (c->server_name[0] == '\0' &&
@@ -278,6 +279,10 @@ fcgi_parse_params(uint8_t *buf, uint16_t n, struct request *c, uint16_t id)
 			c->server_name[val_len] = '\0';
 		}
 
+		if (name_len == 5 &&
+		    strncmp(buf, "HTTPS", 5) == 0)
+			c->https = 1;
+
 		buf += name_len + val_len;
 		n -= name_len - val_len;
 	}
@@ -289,6 +294,21 @@ fcgi_timeout(int fd, short events, void *arg)
 	fcgi_cleanup_request((struct request*) arg);
 }
 
+int
+fcgi_puts(struct template *tp, const char *str)
+{
+	if (str == NULL)
+		return 0;
+	return fcgi_gen_binary_response(tp->tp_arg, str, strlen(str));
+}
+
+int
+fcgi_putc(struct template *tp, int ch)
+{
+	uint8_t c = ch;
+	return fcgi_gen_binary_response(tp->tp_arg, &c, 1);
+}
+
 int
 fcgi_vprintf(struct request *c, const char *fmt, va_list ap)
 {
@@ -485,6 +505,7 @@ fcgi_cleanup_request(struct request *c)
 		event_del(&c->ev);
 
 	close(c->fd);
+	template_free(c->tp);
 	gotweb_free_transport(c->t);
 	free(c);
 }
