@@ -290,6 +290,19 @@ const struct got_error *got_object_blob_read_block(size_t *,
 /* Rewind an open blob's data stream back to the beginning. */
 void got_object_blob_rewind(struct got_blob_object *);
 
+/*
+ * Heuristic to check whether the blob contains binary data.  Rewinds
+ * the blob's data stream back after the header.
+ */
+const struct got_error *got_object_blob_is_binary(int *,
+    struct got_blob_object *);
+
+/*
+ * getline(3) for blobs.
+ */
+const struct got_error *got_object_blob_getline(char **, ssize_t *,
+    size_t *, struct got_blob_object *);
+
 /*
  * Read the entire content of a blob and write it to the specified file.
  * Flush and rewind the file as well. Indicate the amount of bytes
