@@ -43,6 +43,7 @@
 	 * The autoconf test for strerror_r is broken in current versions
 	 * of autoconf: https://savannah.gnu.org/support/?110367
 	 */
+char *__xpg_strerror_r(int, char *, size_t);
 #define strerror_r __xpg_strerror_r
 #endif
 
@@ -241,6 +242,8 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_REF_BUSY, "reference cannot be updated; please try again" },
 	{ GOT_ERR_COMMIT_BAD_AUTHOR, "commit author formatting would "
 	    "make Git unhappy" },
+	{ GOT_ERR_UID, "bad user ID" },
+	{ GOT_ERR_GID, "bad group ID" },
 };
 
 static struct got_custom_error {
