@@ -49,59 +49,14 @@ catch_sigint(int signo)
 	sigint_received = 1;
 }
 
-static const struct got_error *
-read_tree_object(struct got_parsed_tree_entry **entries, int *nentries,
-    uint8_t **p, FILE *f, struct got_object_id *expected_id)
-{
-	const struct got_error *err = NULL;
-	struct got_object *obj = NULL;
-	size_t len;
-	struct got_inflate_checksum csum;
-	SHA1_CTX sha1_ctx;
-	struct got_object_id id;
-
-	SHA1Init(&sha1_ctx);
-	memset(&csum, 0, sizeof(csum));
-	csum.output_sha1 = &sha1_ctx;
-
-	err = got_inflate_to_mem(p, &len, NULL, &csum, f);
-	if (err)
-		return err;
-
-	SHA1Final(id.sha1, &sha1_ctx);
-	if (memcmp(expected_id->sha1, id.sha1, SHA1_DIGEST_LENGTH) != 0) {
-		char buf[SHA1_DIGEST_STRING_LENGTH];
-		err = got_error_fmt(GOT_ERR_OBJ_CSUM,
-		    "checksum failure for object %s",
-		    got_sha1_digest_to_str(expected_id->sha1, buf,
-		    sizeof(buf)));
-		goto done;
-	}
-
-	err = got_object_parse_header(&obj, *p, len);
-	if (err)
-		goto done;
-
-	if (len < obj->hdrlen + obj->size) {
-		err = got_error(GOT_ERR_BAD_OBJ_DATA);
-		goto done;
-	}
-
-	/* Skip object header. */
-	len -= obj->hdrlen;
-	err = got_object_parse_tree(entries, nentries, *p + obj->hdrlen, len);
-done:
-	if (obj)
-		got_object_close(obj);
-	return err;
-}
-
 int
 main(int argc, char *argv[])
 {
 	const struct got_error *err = NULL;
 	struct imsgbuf ibuf;
 	size_t datalen;
+	struct got_parsed_tree_entry *entries = NULL;
+	size_t nentries = 0, nentries_alloc = 0;
 
 	signal(SIGINT, catch_sigint);
 
@@ -130,9 +85,6 @@ main(int argc, char *argv[])
 
 	for (;;) {
 		struct imsg imsg;
-		FILE *f = NULL;
-		struct got_parsed_tree_entry *entries = NULL;
-		int nentries = 0;
 		uint8_t *buf = NULL;
 		struct got_object_id expected_id;
 
@@ -169,33 +121,22 @@ main(int argc, char *argv[])
 		}
 
 		/* Always assume file offset zero. */
-		f = fdopen(imsg.fd, "rb");
-		if (f == NULL) {
-			err = got_error_from_errno("fdopen");
-			goto done;
-		}
-
-		err = read_tree_object(&entries, &nentries, &buf, f,
-		    &expected_id);
+		err = got_object_read_tree(&entries, &nentries, &nentries_alloc,
+		    &buf, imsg.fd, &expected_id);
 		if (err)
 			goto done;
 
 		err = got_privsep_send_tree(&ibuf, entries, nentries);
 done:
-		free(entries);
 		free(buf);
-		if (f) {
-			if (fclose(f) == EOF && err == NULL)
-				err = got_error_from_errno("fclose");
-		} else if (imsg.fd != -1) {
-			if (close(imsg.fd) == -1 && err == NULL)
-				err = got_error_from_errno("close");
-		}
+		if (imsg.fd != -1 && close(imsg.fd) == -1 && err == NULL)
+			err = got_error_from_errno("close");
 		imsg_free(&imsg);
 		if (err)
 			break;
 	}
 
+	free(entries);
 	imsg_clear(&ibuf);
 	if (err) {
 		if (!sigint_received && err->code != GOT_ERR_PRIVSEP_PIPE) {
